@@ -2,13 +2,14 @@
 
 use crate::{
     state::{
+        enums::MintMaxVoteWeightSource,
         governance::{
             get_account_governance_address, get_mint_governance_address,
             get_program_governance_address, get_token_governance_address, GovernanceConfig,
         },
         proposal::get_proposal_address,
         proposal_instruction::{get_proposal_instruction_address, InstructionData},
-        realm::{get_governing_token_holding_address, get_realm_address},
+        realm::{get_governing_token_holding_address, get_realm_address, RealmConfigArgs},
         signatory_record::get_signatory_record_address,
         token_owner_record::get_token_owner_record_address,
         vote_record::get_vote_record_address,
@@ -41,7 +42,7 @@ pub enum GovernanceInstruction {
     /// Creates Governance Realm account which aggregates governances for given Community Mint and optional Council Mint
     ///
     /// 0. `[writable]` Governance Realm account. PDA seeds:['governance',name]
-    /// 1. `[]` Realm authority    
+    /// 1. `[]` Realm authority
     /// 2. `[]` Community Token Mint
     /// 3. `[writable]` Community Token Holding account. PDA seeds: ['governance',realm,community_mint]
     ///     The account will be created with the Realm PDA as its owner
@@ -49,13 +50,18 @@ pub enum GovernanceInstruction {
     /// 5. `[]` System
     /// 6. `[]` SPL Token
     /// 7. `[]` Sysvar Rent
-    /// 8. `[]` Council Token Mint - optional
-    /// 9. `[writable]` Council Token Holding account - optional. . PDA seeds: ['governance',realm,council_mint]
+    /// 8. `[]` Realm custodian - optional    
+    /// 9. `[]` Council Token Mint - optional
+    /// 10. `[writable]` Council Token Holding account - optional unless council is used. PDA seeds: ['governance',realm,council_mint]
     ///     The account will be created with the Realm PDA as its owner
     CreateRealm {
         #[allow(dead_code)]
         /// UTF-8 encoded Governance Realm name
         name: String,
+
+        #[allow(dead_code)]
+        /// Realm config args     
+        config_args: RealmConfigArgs,
     },
 
     /// Deposits governing tokens (Community or Council) to Governance Realm and establishes your voter weight to be used for voting within the Realm
@@ -373,18 +379,37 @@ pub enum GovernanceInstruction {
         /// New realm authority or None to remove authority
         new_realm_authority: Option<Pubkey>,
     },
+
+    /// Sets realm config
+    ///   0. `[writable]` Realm account
+    ///   1. `[signer]`  Realm authority    
+    ///   2. `[]` Realm custodian - optional    
+    ///   3. `[]` Council Token Mint - optional
+    ///       Note: In the current version it's only possible to remove council mint (set it to None)
+    ///       After setting council to None it won't be possible to withdraw the tokens from the Realm any longer
+    ///       If that's required then it must be done before executing this instruction
+    ///   4. `[writable]` Council Token Holding account - optional unless council is used. PDA seeds: ['governance',realm,council_mint]
+    ///       The account will be created with the Realm PDA as its owner
+    SetRealmConfig {
+        #[allow(dead_code)]
+        /// Realm config args
+        config_args: RealmConfigArgs,
+    },
 }
 
 /// Creates CreateRealm instruction
+#[allow(clippy::too_many_arguments)]
 pub fn create_realm(
     program_id: &Pubkey,
     // Accounts
     realm_authority: &Pubkey,
     community_token_mint: &Pubkey,
     payer: &Pubkey,
+    realm_custodian: Option<Pubkey>,
     council_token_mint: Option<Pubkey>,
     // Args
     name: String,
+    community_mint_max_vote_weight_source: MintMaxVoteWeightSource,
 ) -> Instruction {
     let realm_address = get_realm_address(program_id, &name);
     let community_token_holding_address =
@@ -401,15 +426,32 @@ pub fn create_realm(
         AccountMeta::new_readonly(sysvar::rent::id(), false),
     ];
 
-    if let Some(council_token_mint) = council_token_mint {
+    let use_custodian = if let Some(realm_custodian) = realm_custodian {
+        accounts.push(AccountMeta::new_readonly(realm_custodian, false));
+        true
+    } else {
+        false
+    };
+
+    let use_council_mint = if let Some(council_token_mint) = council_token_mint {
         let council_token_holding_address =
             get_governing_token_holding_address(program_id, &realm_address, &council_token_mint);
 
         accounts.push(AccountMeta::new_readonly(council_token_mint, false));
         accounts.push(AccountMeta::new(council_token_holding_address, false));
-    }
+        true
+    } else {
+        false
+    };
 
-    let instruction = GovernanceInstruction::CreateRealm { name };
+    let instruction = GovernanceInstruction::CreateRealm {
+        config_args: RealmConfigArgs {
+            use_council_mint,
+            use_custodian,
+            community_mint_max_vote_weight_source,
+        },
+        name,
+    };
 
     Instruction {
         program_id: *program_id,
@@ -1109,3 +1151,52 @@ pub fn set_realm_authority(
         data: instruction.try_to_vec().unwrap(),
     }
 }
+
+/// Creates SetRealmConfig instruction
+pub fn set_realm_config(
+    program_id: &Pubkey,
+    // Accounts
+    realm: &Pubkey,
+    realm_authority: &Pubkey,
+    council_token_mint: Option<Pubkey>,
+    realm_custodian: Option<Pubkey>,
+    // Args
+    community_mint_max_vote_weight_source: MintMaxVoteWeightSource,
+) -> Instruction {
+    let mut accounts = vec![
+        AccountMeta::new(*realm, false),
+        AccountMeta::new_readonly(*realm_authority, true),
+    ];
+
+    let use_custodian = if let Some(realm_custodian) = realm_custodian {
+        accounts.push(AccountMeta::new_readonly(realm_custodian, false));
+        true
+    } else {
+        false
+    };
+
+    let use_council_mint = if let Some(council_token_mint) = council_token_mint {
+        let council_token_holding_address =
+            get_governing_token_holding_address(program_id, realm, &council_token_mint);
+
+        accounts.push(AccountMeta::new_readonly(council_token_mint, false));
+        accounts.push(AccountMeta::new(council_token_holding_address, false));
+        true
+    } else {
+        false
+    };
+
+    let instruction = GovernanceInstruction::SetRealmConfig {
+        config_args: RealmConfigArgs {
+            use_council_mint,
+            use_custodian,
+            community_mint_max_vote_weight_source,
+        },
+    };
+
+    Instruction {
+        program_id: *program_id,
+        accounts,
+        data: instruction.try_to_vec().unwrap(),
+    }
+}
