@@ -5,7 +5,7 @@ use crate::{
     state::{
         enums::GovernanceAccountType,
         governance::{
-            assert_is_valid_governance_config, get_program_governance_address_seeds,
+            assert_valid_create_governance_args, get_program_governance_address_seeds,
             GovernanceConfig,
         },
     },
@@ -36,22 +36,25 @@ pub fn process_create_program_governance(
     let realm_info = next_account_info(account_info_iter)?; // 0
     let program_governance_info = next_account_info(account_info_iter)?; // 0
 
-    let governed_program_data_info = next_account_info(account_info_iter)?; // 1
-    let governed_program_upgrade_authority_info = next_account_info(account_info_iter)?; // 2
+    let governed_program_info = next_account_info(account_info_iter)?; // 1
+    let governed_program_data_info = next_account_info(account_info_iter)?; // 2
+    let governed_program_upgrade_authority_info = next_account_info(account_info_iter)?; // 3
 
-    let payer_info = next_account_info(account_info_iter)?; // 3
-    let bpf_upgrade_loader_info = next_account_info(account_info_iter)?; // 4
+    let payer_info = next_account_info(account_info_iter)?; // 4
+    let bpf_upgrade_loader_info = next_account_info(account_info_iter)?; // 5
 
-    let system_info = next_account_info(account_info_iter)?; // 5
+    let system_info = next_account_info(account_info_iter)?; // 6
 
-    let rent_sysvar_info = next_account_info(account_info_iter)?; // 6
+    let rent_sysvar_info = next_account_info(account_info_iter)?; // 7
     let rent = &Rent::from_account_info(rent_sysvar_info)?;
 
-    assert_is_valid_governance_config(program_id, &config, realm_info)?;
+    assert_valid_create_governance_args(program_id, &config, realm_info)?;
 
     let program_governance_data = Governance {
         account_type: GovernanceAccountType::ProgramGovernance,
-        config: config.clone(),
+        realm: *realm_info.key,
+        governed_account: *governed_program_info.key,
+        config,
         proposals_count: 0,
         reserved: [0; 8],
     };
@@ -60,7 +63,7 @@ pub fn process_create_program_governance(
         payer_info,
         program_governance_info,
         &program_governance_data,
-        &get_program_governance_address_seeds(&config.realm, &config.governed_account),
+        &get_program_governance_address_seeds(realm_info.key, governed_program_info.key),
         program_id,
         system_info,
         rent,
@@ -68,7 +71,7 @@ pub fn process_create_program_governance(
 
     if transfer_upgrade_authority {
         set_program_upgrade_authority(
-            &config.governed_account,
+            governed_program_info.key,
             governed_program_data_info,
             governed_program_upgrade_authority_info,
             program_governance_info,
@@ -76,7 +79,7 @@ pub fn process_create_program_governance(
         )?;
     } else {
         assert_program_upgrade_authority_is_signer(
-            &config.governed_account,
+            governed_program_info.key,
             governed_program_data_info,
             governed_program_upgrade_authority_info,
         )?;
