@@ -11,9 +11,10 @@ use solana_program::{
 use crate::{
     error::GovernanceError,
     state::{
-        enums::{GovernanceAccountType, MintMaxVoteWeightSource},
+        enums::GovernanceAccountType,
         realm::{
-            get_governing_token_holding_address_seeds, get_realm_address_seeds, Realm, RealmConfig,
+            assert_valid_realm_config_args, get_governing_token_holding_address_seeds,
+            get_realm_address_seeds, Realm, RealmConfig, RealmConfigArgs,
         },
     },
     tools::{
@@ -26,6 +27,7 @@ pub fn process_create_realm(
     program_id: &Pubkey,
     accounts: &[AccountInfo],
     name: String,
+    config_args: RealmConfigArgs,
 ) -> ProgramResult {
     let account_info_iter = &mut accounts.iter();
 
@@ -44,6 +46,8 @@ pub fn process_create_realm(
         return Err(GovernanceError::RealmAlreadyExists.into());
     }
 
+    assert_valid_realm_config_args(&config_args)?;
+
     create_spl_token_account_signed(
         payer_info,
         governance_token_holding_info,
@@ -57,11 +61,16 @@ pub fn process_create_realm(
         rent,
     )?;
 
-    let council_token_mint_address = if let Ok(council_token_mint_info) =
-        next_account_info(account_info_iter)
-    // 7
-    {
-        let council_token_holding_info = next_account_info(account_info_iter)?; //8
+    let realm_custodian = if config_args.use_custodian {
+        let realm_custodian_info = next_account_info(account_info_iter)?;
+        Some(*realm_custodian_info.key)
+    } else {
+        None
+    };
+
+    let council_token_mint_address = if config_args.use_council_mint {
+        let council_token_mint_info = next_account_info(account_info_iter)?;
+        let council_token_holding_info = next_account_info(account_info_iter)?;
 
         create_spl_token_account_signed(
             payer_info,
@@ -91,8 +100,9 @@ pub fn process_create_realm(
         config: RealmConfig {
             council_mint: council_token_mint_address,
             reserved: [0; 8],
-            custodian: Some(*realm_authority_info.key),
-            community_mint_max_vote_weight_source: MintMaxVoteWeightSource::Percentage(100),
+            custodian: realm_custodian,
+            community_mint_max_vote_weight_source: config_args
+                .community_mint_max_vote_weight_source,
         },
     };
 
